require 'spec_helper'

describe 'Strutta API Wrapper' do

  before { WebMock.disable_net_connect!(allow_localhost: true, allow: /codeclimate/) }
  after { WebMock.allow_net_connect! }

  let(:token) { 'dc1479c52801fbfa0975947de092a1e7' }
  let(:host) { 'http://api.strutta.dev:4000' }
  let(:path) { '/v2/' }
  let(:strutta) { Strutta::API.new token, host, path }

  let(:new_game) { strutta.games.create }
  let(:game) { strutta.games(VALID_GAME_01[:id]) }

  let(:judgments) do
    [
      {
        entry_id: 1234,
        rank: 1,
        metadata: {
          comments: 'First place'
        }
      },
      {
        entry_id: 1235,
        rank: 2,
        metadata: {
          comments: 'Second place'
        }
      },
      {
        entry_id: 1236,
        rank: 3,
        metadata: {
          comments: 'Third place'
        }
      }
    ]
  end

  describe 'games(:id).judging' do
    context 'when no id is given' do

      context '.create' do
        context 'with valid parameters' do
          before do
            stub_request(:post, /.*judging/)
             .to_return(status: 201, body: JUDGING_RESPONSE.to_json, headers: {})
          end
          it 'should return a new flow hash' do
            judging = strutta.games(333).judging.create(judgments)
            expect(judging['message']).to eq JUDGING_RESPONSE[:message]
          end
        end
      end

      context '.get' do
        before do
          stub_request(:get, /.*judging/)
           .to_return(status: 200, body: JUDGING_GET_RESPONSE.to_json, headers: {})
        end
        it 'should throw disabled endpoint error' do
          judging = strutta.games(333).judging.get(round_id: JUDGING_ROUND[:id])
          expect(judging['round_id'].is_a? Integer).to be true
          expect(judging['judging'].is_a? Array).to be true
          expect(judging['judging'].first['judgments'].is_a? Array).to be true
        end
      end

      context '.all' do
        it 'should throw disabled endpoint error' do
          expect { game.judging.all({}) }.to raise_error(Strutta::Errors::DisabledEndpointError)
        end
      end

      context '.update' do
        it 'should throw disabled endpoint error' do
          expect { game.judging.update({}) }.to raise_error(Strutta::Errors::DisabledEndpointError)
        end
      end

      context '.delete' do
        it 'should throw disabled endpoint error' do
          expect { game.judging.delete({}) }.to raise_error(Strutta::Errors::DisabledEndpointError)
        end
      end
    end

    context 'when id is given' do
      context '.all' do
        it 'should throw disabled endpoint error' do
          expect { game.judging(1).all({}) }.to raise_error(Strutta::Errors::DisabledEndpointError)
        end
      end

      context '.update' do
        it 'should throw disabled endpoint error' do
          expect { game.judging(1).update({}) }.to raise_error(Strutta::Errors::DisabledEndpointError)
        end
      end

      context '.delete' do
        it 'should throw disabled endpoint error' do
          expect { game.judging(1).delete({}) }.to raise_error(Strutta::Errors::DisabledEndpointError)
        end
      end

      context '.create' do
        it 'should throw error' do
          expect { game.judging(1).create }.to raise_error(Strutta::Errors::ObjectIDNotAllowed)
        end
      end

      context '.get' do
        it 'should throw error' do
          expect { game.judging(1).create }.to raise_error(Strutta::Errors::ObjectIDNotAllowed)
        end
      end
    end
  end
end
